//
// Tutorial na SDL_image
// pro kompilaci v Linuxu :  gcc `sdl-config --libs --cflags` -lSDL_image -L/usr/X11R6/lib -lGL -lGLU lessonSDL_image.cpp    
//
//                  jinde :  nezapomente krome OpenGL a SDL prilinkovat i SDL_image
//
// Bernard Lidicky - 2berny@seznam.cz, www.hippo.nipax.cz
// pro
// 
// www.nehe.opengl.cz
// 


#include <SDL.h>        // hlavni knihovna
#include <SDL_opengl.h> // include za nas OpenGL
#include <SDL_image.h>  // aby jsme mohli pouzivat funkci IMG_Load

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

GLuint gl_texture; // nase texturka

// Funkce na nacteni bitmapy
SDL_Surface* LoadBitmap(const char *filename)
{
   Uint8 *rowhi, *rowlo; // ukazatele na prohazovani radku
   Uint8 *tmpbuf, tmpch; // docasna pamet
   SDL_Surface *image;   // nacteny obrazek
   int i, j;             // ridici promene pro cykly


   // nacteni obrazkovych dat
   image = IMG_Load(filename);
   if ( image == NULL ) {
      fprintf(stderr, "Nepodarilo se nacist %s: %s\n", filename, SDL_GetError());
      return(NULL);
   }

    // GL surfaces jsou vzhuru nohama - budeme nasi bitmapu muset prevratit
   tmpbuf = (Uint8 *)malloc(image->pitch);

   if ( tmpbuf == NULL ) {
       fprintf(stderr, "Nedostatek pameti\n");
       return NULL;
   }

   // nastaveni prvniho a posledniho radku
   rowhi = (Uint8 *)image->pixels;
   rowlo = rowhi + (image->h * image->pitch) - image->pitch;
   
   for ( i = 0; i < image->h/2; i++ ) {
	   
      // prevraceni BGR na RGB
      if (image->format->Bshift == 0)
         for ( j = 0; j < image->w; j++ ) {
            tmpch        = rowhi[j*3];
            rowhi[j*3]   = rowhi[j*3+2];
            rowhi[j*3+2] = tmpch;
            tmpch        = rowlo[j*3];
            rowlo[j*3]   = rowlo[j*3+2];
            rowlo[j*3+2] = tmpch;
         }

      // prohozeni radku
      memcpy(tmpbuf, rowhi, image->pitch);
      memcpy(rowhi,  rowlo, image->pitch);
      memcpy(rowlo, tmpbuf, image->pitch);
      
      // posun ikazatelu na radky
      rowhi += image->pitch;
      rowlo -= image->pitch;
   }

   // uklidime
   free(tmpbuf);

   return image;
}

// Funkce pro konverzi bitmapy do textury
GLuint CreateTexture(const char* file, int min_filter, int mag_filter, bool mipmaps)
{
   // nacteni obrazku
   SDL_Surface *surface;
   surface = LoadBitmap(file);
   if (surface == NULL) return 0;

   // vytvoreni objektu textury
   GLuint texture;

   // vygenerovani textury
   glGenTextures(1, &texture);

   // nastaveni textury
   glBindTexture(GL_TEXTURE_2D, texture);

   // predani parametru
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mag_filter);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, min_filter);

   // pridani barevne mapy (obrazku) - pripadne generovani mip map
   if (mipmaps) 
      gluBuild2DMipmaps(GL_TEXTURE_2D, 3, surface->w, surface->h, GL_RGB, GL_UNSIGNED_BYTE, surface->pixels);
   else
      glTexImage2D(GL_TEXTURE_2D, 0, 3, surface->w, surface->h, 0, GL_RGB, GL_UNSIGNED_BYTE, surface->pixels);

   // uklid - smazani SDL_Surface
   SDL_FreeSurface(surface);
   surface = NULL;
   
   return texture;
}


// nakresli scenu
void RenderScene()
{
   static float rott = 0.0f;
   
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 
   glLoadIdentity();

   glBindTexture(GL_TEXTURE_2D, gl_texture);

   glTranslatef(0.0f, 0.0f, -6.0f);
   
   glRotatef(rott, -1.0,  0.0,  0.0); 
   glRotatef(rott,  0.0,  1.0,  0.0); 
   glRotatef(rott,  0.0,  0.0, -1.0);   
   
   glBegin(GL_POLYGON);
      glTexCoord2f(0.0f, 0.0f); glVertex3f( -1.0f,  0.0f, -1.0);
      glTexCoord2f(1.0f, 0.0f); glVertex3f( -1.0f,  0.0f,  1.0);
      glTexCoord2f(1.0f, 1.0f); glVertex3f(  1.0f,  0.0f,  1.0);
      glTexCoord2f(0.0f, 1.0f); glVertex3f(  1.0f,  0.0f, -1.0);
   glEnd();
	   
   rott += 0.1;  

   SDL_GL_SwapBuffers(); 
}

// Inicializace SDL
void InitSDL()
{
   // Inicializace SDL grafiky a SDL Timeru 
   if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
      fprintf(stderr, "Nepodarilo se inicializovat SDL: %s\n", SDL_GetError());
      exit(1);
   }

   // nastaveni funkce pri volani exit();
   atexit(SDL_Quit);

   // chceme doublebuffering s openGL
   SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );


   // Nastavime velikost a styl okna
   unsigned int flags = SDL_OPENGL;// | SDL_FULLSCREEN; // - pripadne pridame FullScreen

   // vytvorime okno
   if ( SDL_SetVideoMode(640, 480, 0, flags) == NULL ) {
      fprintf(stderr, "Nepodarilo se vytvorit OpenGL okno : %s\n", SDL_GetError());
      SDL_Quit();
      exit(2);
   }
	
   // Nastaveni titulku okna
   SDL_WM_SetCaption("Lesson - SDL_image", NULL);
}

// Nastaveni zakladnich parametru pro OpenGL
void InitOpenGL()	        
{
   glViewport(0, 0, 640, 480);
   glEnable(GL_TEXTURE_2D);               // Povolime textury
   glClearColor(0.0f, 0.0f, 0.0f, 0.0f);  // Nastavime barvu pozadi na cernou
   glClearDepth(1.0);                     // povolime mazani hloubkove bufferu
   glDepthFunc(GL_LESS);
   glEnable(GL_DEPTH_TEST);               // Povolime testovani hloubky
   glShadeModel(GL_SMOOTH);

   glEnable(GL_LIGHT0);		
   glEnable(GL_COLOR_MATERIAL);	

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();

   gluPerspective(45.0f,(GLfloat)640/(GLfloat)480,0.1f,1000.0f);

   glMatrixMode(GL_MODELVIEW);

   // nacteme texturu
   if ( (gl_texture = CreateTexture("test.jpg", GL_LINEAR, GL_LINEAR, false)) == 0) {
      fprintf(stderr, "Nepodarilo se vytvorit texturu\n");
      exit(1);
   }
}

// odinicializuje vse co zbylo
void ShutDownApp()
{
   // smazeme texturu
   glDeleteTextures(1, &gl_texture);
   
   // ukoncime SDL
   SDL_Quit();
}

// Zpracovava udalosti. 
bool ProcessEvents()
{
   SDL_Event event;
   while ( SDL_PollEvent(&event) ) {
      switch (event.type) {
      
      case SDL_QUIT :       // uzivatel si preje ukoncit aplikace
         return false;
         break;
	 
      case SDL_KEYDOWN :    // stisknuta klavesa
         switch (event.key.keysym.sym) {
         case SDLK_ESCAPE :
            return false;
            break;
         }
         break;
	 
      case SDL_VIDEOEXPOSE : // potreba prekreslit okno
          RenderScene();
          break;
      }
   }
   return true;
}


// hlavni funkce main
int main(int argc, char ** argv) 
{
   
   InitSDL(); // inicializace SDL
 
   InitOpenGL(); // inicializace OpenGL

   // dokud se dari zpracovavat message, tak se snazime fungovat
   while ( ProcessEvents() ) {
      RenderScene();
      
   }
   
   ShutDownApp();
   
   return 0;
}
